@@ -6,7 +6,8 @@ from __future__ import unicode_literals
 import numpy as np
 
 import onnx
-from .base import Base, expect
+from ..base import Base
+from . import expect
 
 
 class Slice(Base):
@@ -45,6 +46,40 @@ class Slice(Base):
         expect(node, inputs=[x], outputs=[y],
                name='test_slice_neg')
 
+    @staticmethod
+    def export_slice_start_out_of_bounds():
+        node = onnx.helper.make_node(
+            'Slice',
+            inputs=['x'],
+            outputs=['y'],
+            axes=[1],
+            starts=[1000],
+            ends=[1000],
+        )
+
+        x = np.random.randn(20, 10, 5).astype(np.float32)
+        y = x[:, 1000:1000]
+
+        expect(node, inputs=[x], outputs=[y],
+               name='test_slice_start_out_of_bounds')
+
+    @staticmethod
+    def export_slice_end_out_of_bounds():
+        node = onnx.helper.make_node(
+            'Slice',
+            inputs=['x'],
+            outputs=['y'],
+            axes=[1],
+            starts=[1],
+            ends=[1000],
+        )
+
+        x = np.random.randn(20, 10, 5).astype(np.float32)
+        y = x[:, 1:1000]
+
+        expect(node, inputs=[x], outputs=[y],
+               name='test_slice_end_out_of_bounds')
+
     @staticmethod
     def export_slice_default_axes():
         node = onnx.helper.make_node(
@@ -59,4 +94,4 @@ class Slice(Base):
         y = x[:, :, 3:4]
 
         expect(node, inputs=[x], outputs=[y],
-               name='test_default_axes')
+               name='test_slice_default_axes')
