@@ -4,11 +4,9 @@
 #include "onnx/defs/schema.h"
 #include <functional>
 
-using namespace onnx;
+using namespace ONNX_NAMESPACE;
 
-using AttrType = onnx::OpSchema::AttrType;
-
-namespace onnx {
+namespace ONNX_NAMESPACE {
 
 const char* kBroadcastDoc = R"DOC(
 If necessary the right-hand-side argument will be broadcasted to match the
@@ -39,10 +37,12 @@ Performs element-wise binary {name} (with limited broadcast support).
     schema.SetDoc(doc);
     schema.Attr("broadcast",
                 "Pass 1 to enable broadcasting",
-                AttrType::INT);
+                AttributeProto::INT,
+                static_cast<int64_t>(0));
     schema.Attr("axis",
                 "If set, defines the broadcast dimensions. See doc for details.",
-                AttrType::INT);
+                AttributeProto::INT,
+                OPTIONAL);
     schema.Input(
         0,
         "A",
@@ -58,13 +58,13 @@ Performs element-wise binary {name} (with limited broadcast support).
   };
 }
 
-std::function<void(OpSchema&)> SoftmaxFamilyDocGenerator(const char* name) {
+std::function<void(OpSchema&)> SoftmaxFamilyDocGenerator(const char* name, const char* description) {
   return [=](OpSchema& schema) {
     std::string doc = R"DOC(
-The operator computes the {name} normalized values for each layer in the batch
+The operator computes the {name} ({description}) values for each layer in the batch
  of the given input. The input is a 2-D tensor (Tensor<float>) of size
 (batch_size x input_feature_dimensions). The output tensor has the same shape
-and contains the {name} normalized values of the corresponding input.
+and contains the {name} values of the corresponding input.
 
 X does not need to explicitly be a 2D vector; rather, it will be
 coerced into one. For an arbitrary n-dimensional tensor
@@ -78,50 +78,42 @@ Each of these dimensions must be matched correctly, or else the operator
 will throw errors.
 )DOC";
     ReplaceAll(doc, "{name}", name);
+    ReplaceAll(doc, "{description}", description);
     schema.SetDoc(doc);
     schema.Attr("axis",
         "(int) default to 1; describes the axis of the inputs when coerced "
         "to 2D; defaults to one because the 0th axis most likely describes "
         "the batch_size",
-        AttrType::INT);
+        AttributeProto::INT,
+        static_cast<int64_t>(1));
     schema.Input(0, "input",
          "The input tensor that's coerced into a 2D matrix of size (NxD) "
          "as described above.", "T");
-    schema.Output(0, "output", "The softmax normalized output values with the same "
+    schema.Output(0, "output", "The output values with the same "
           "shape as input tensor.", "T");
     schema.TypeConstraint("T", { "tensor(float16)", "tensor(float)", "tensor(double)" },
         "Constrain input and output types to float tensors.");
   };
 }
 
-OPERATOR_SCHEMA(Add)
-    .NumInputs(2)
-    .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(Add)
     .AllowConsumed({{0, 0}, {1, 0}})
     .FillUsing(MathDocGenerator("addition"));
 
-OPERATOR_SCHEMA(Sub)
-    .NumInputs(2)
-    .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(Sub)
     .AllowConsumed({{0, 0}, {1, 0}})
     .FillUsing(MathDocGenerator("subtraction"));
 
-OPERATOR_SCHEMA(Mul)
-    .NumInputs(2)
-    .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(Mul)
     .AllowConsumed({{0, 0}, {1, 0}})
     .FillUsing(MathDocGenerator("multiplication"));
 
-OPERATOR_SCHEMA(Div)
-    .NumInputs(2)
-    .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(Div)
     .AllowConsumed({{0, 0}, {1, 0}})
     .FillUsing(MathDocGenerator("division"));
-}  // namespace onnx
+}  // namespace ONNX_NAMESPACE
 
-OPERATOR_SCHEMA(Neg)
-    .NumInputs(1)
-    .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(Neg)
     .AllowConsumed({{0, 0}})
     .SetDoc(R"DOC(
 Neg takes one input data (Tensor<T>) and produces one output data
@@ -133,9 +125,7 @@ the tensor elementwise.
     .TypeConstraint("T", { "tensor(float16)", "tensor(float)", "tensor(double)" },
         "Constrain input and output types to float tensors.");
 
-OPERATOR_SCHEMA(Abs)
-    .NumInputs(1)
-    .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(Abs)
     .AllowConsumed({{0, 0}})
     .SetDoc(R"DOC(
 Absolute takes one input data (Tensor<T>) and produces one output data
@@ -147,9 +137,7 @@ the tensor elementwise.
     .TypeConstraint("T", { "tensor(float16)", "tensor(float)", "tensor(double)" },
         "Constrain input and output types to float tensors.");
 
-OPERATOR_SCHEMA(Reciprocal)
-    .NumInputs(1)
-    .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(Reciprocal)
     .AllowConsumed({{0, 0}})
     .SetDoc(R"DOC(
 Reciprocal takes one input data (Tensor<T>) and produces one output data
@@ -161,9 +149,7 @@ the tensor elementwise.
     .TypeConstraint("T", { "tensor(float16)", "tensor(float)", "tensor(double)" },
         "Constrain input and output types to float tensors.");
 
-OPERATOR_SCHEMA(Floor)
-    .NumInputs(1)
-    .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(Floor)
     .AllowConsumed({{0, 0}})
     .SetDoc(R"DOC(
 Floor takes one input data (Tensor<T>) and produces one output data
@@ -175,9 +161,7 @@ the tensor elementwise.
     .TypeConstraint("T", { "tensor(float16)", "tensor(float)", "tensor(double)" },
         "Constrain input and output types to float tensors.");
 
-OPERATOR_SCHEMA(Ceil)
-    .NumInputs(1)
-    .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(Ceil)
     .AllowConsumed({{0, 0}})
     .SetDoc(R"DOC(
 Ceil takes one input data (Tensor<T>) and produces one output data
@@ -189,9 +173,7 @@ the tensor elementwise.
     .TypeConstraint("T", { "tensor(float16)", "tensor(float)", "tensor(double)" },
         "Constrain input and output types to float tensors.");
 
-OPERATOR_SCHEMA(Sqrt)
-    .NumInputs(1)
-    .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(Sqrt)
     .AllowConsumed({{0, 0}})
     .SetDoc(R"DOC(
 Square root takes one input data (Tensor<T>) and produces one output data
@@ -203,9 +185,7 @@ the tensor elementwise. If x is negative, then it will return NaN.
     .TypeConstraint("T", { "tensor(float16)", "tensor(float)", "tensor(double)" },
         "Constrain input and output types to float tensors.");
 
-OPERATOR_SCHEMA(Relu)
-  .NumInputs(1)
-  .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(Relu)
   .AllowConsumed({{0, 0}})
   .SetDoc(R"DOC(
 Relu takes one input data (Tensor<T>) and produces one output data
@@ -217,12 +197,11 @@ the tensor elementwise.
   .TypeConstraint("T", { "tensor(float16)", "tensor(float)", "tensor(double)" },
         "Constrain input and output types to float tensors.");
 
-OPERATOR_SCHEMA(LeakyRelu)
-    .NumInputs(1)
-    .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(LeakyRelu)
     .Attr("alpha",
-          "Coefficient of leakage",
-          AttrType::FLOAT)
+          "Coefficient of leakage default to 0.01.",
+          AttributeProto::FLOAT,
+          0.01f)
     .AllowConsumed({{0, 0}})
     .SetDoc(R"DOC(
 LeakyRelu takes input data (Tensor<T>) and an argument alpha, and produces one
@@ -234,16 +213,16 @@ output data (Tensor<T>) where the function `f(x) = alpha * x for x < 0`,
     .TypeConstraint("T", { "tensor(float16)", "tensor(float)", "tensor(double)" },
         "Constrain input and output types to float tensors.");
 
-OPERATOR_SCHEMA(Selu)
-    .NumInputs(1)
-    .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(Selu)
     .AllowConsumed({{0, 0}})
     .Attr("alpha",
           "Coefficient of SELU default to 1.6732.",
-          AttrType::FLOAT)
+          AttributeProto::FLOAT,
+          1.6732f)
     .Attr("gamma",
           "Coefficient of SELU default to 1.0507.",
-           AttrType::FLOAT)
+           AttributeProto::FLOAT,
+           1.0507f)
     .SetDoc(R"DOC(
 Selu takes one input data (Tensor<T>) and produces one output data
 (Tensor<T>) where the scaled exponential linear unit function,
@@ -255,13 +234,12 @@ is applied to the tensor elementwise.
     .TypeConstraint("T", { "tensor(float16)", "tensor(float)", "tensor(double)" },
         "Constrain input and output types to float tensors.");
 
-OPERATOR_SCHEMA(Elu)
-    .NumInputs(1)
-    .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(Elu)
     .AllowConsumed({{0, 0}})
     .Attr("alpha",
           "Coefficient of ELU default to 1.0.",
-          AttrType::FLOAT)
+          AttributeProto::FLOAT,
+          1.0f)
     .SetDoc(R"DOC(
 Elu takes one input data (Tensor<T>) and produces one output data
 (Tensor<T>) where the function `f(x) = alpha * (exp(x) - 1.) for x <
@@ -273,9 +251,7 @@ Elu takes one input data (Tensor<T>) and produces one output data
     .TypeConstraint("T", { "tensor(float16)", "tensor(float)", "tensor(double)" },
         "Constrain input and output types to float tensors.");
 
-OPERATOR_SCHEMA(Exp)
-    .NumInputs(1)
-    .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(Exp)
     .AllowConsumed({{0, 0}})
     .SetDoc(R"DOC(
 Calculates the exponential of the given input tensor, element-wise.
@@ -289,9 +265,7 @@ Calculates the exponential of the given input tensor, element-wise.
     .TypeConstraint("T", { "tensor(float16)", "tensor(float)", "tensor(double)" },
         "Constrain input and output types to float tensors.");
 
-OPERATOR_SCHEMA(Log)
-    .NumInputs(1)
-    .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(Log)
     .AllowConsumed({{0, 0}})
     .SetDoc(R"DOC(
 Calculates the natural log of the given input tensor, element-wise.
@@ -305,9 +279,7 @@ Calculates the natural log of the given input tensor, element-wise.
     .TypeConstraint("T", { "tensor(float16)", "tensor(float)", "tensor(double)" },
         "Constrain input and output types to float tensors.");
 
-OPERATOR_SCHEMA(Tanh)
-  .NumInputs(1)
-  .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(Tanh)
   .AllowConsumed({{0, 0}})
   .SetDoc(R"DOC(
 Calculates the hyperbolic tangent of the given input tensor element-wise.
@@ -318,24 +290,28 @@ Calculates the hyperbolic tangent of the given input tensor element-wise.
     .TypeConstraint("T", { "tensor(float16)", "tensor(float)", "tensor(double)" },
         "Constrain input and output types to float tensors.");
 
-OPERATOR_SCHEMA(Pow)
-    .NumInputs(2)
-    .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(Pow)
     .SetDoc(R"DOC(
 Pow takes input data (Tensor<T>) and exponent Tensor, and
 produces one output data (Tensor<T>) where the function `f(x) = x^exponent`,
 is applied to the data tensor elementwise.
-)DOC")
+)DOC" + std::string(kBroadcastDoc))
     .Input(0, "X", "Input tensor of any shape, base of the exponent.", "T")
     .Input(1, "Y", "Input tensor of any shape broadcastable to X shape, "
                    "the exponent component.", "T")
+    .Attr("broadcast",
+          "Pass 1 to enable broadcasting",
+          AttributeProto::INT,
+          static_cast<int64_t>(0))
+    .Attr("axis",
+          "If set, defines the broadcast dimensions. See doc for details.",
+          AttributeProto::INT,
+          OPTIONAL)
     .Output(0, "Z", "Output tensor (same size as X)", "T")
     .TypeConstraint("T", { "tensor(float16)", "tensor(float)", "tensor(double)" },
         "Constrain input and output types to float tensors.");
 
-OPERATOR_SCHEMA(PRelu)
-    .NumInputs(2)
-    .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(PRelu)
     .AllowConsumed({{0, 0}})
     .SetDoc(R"DOC(
 
@@ -354,9 +330,7 @@ output data (Tensor<T>) where the function `f(x) = slope * x for x < 0`,
     .TypeConstraint("T", { "tensor(float16)", "tensor(float)", "tensor(double)" },
         "Constrain input and output types to float tensors.");
 
-OPERATOR_SCHEMA(Sigmoid)
-  .NumInputs(1)
-  .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(Sigmoid)
   .AllowConsumed({{0, 0}})
   .SetDoc(R"DOC(
 Sigmoid takes one input data (Tensor<T>) and produces one output data
@@ -368,16 +342,16 @@ tensor elementwise.
     .TypeConstraint("T", { "tensor(float16)", "tensor(float)", "tensor(double)" },
         "Constrain input and output types to float tensors.");
 
-OPERATOR_SCHEMA(HardSigmoid)
-  .NumInputs(1)
-  .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(HardSigmoid)
   .AllowConsumed({{0, 0}})
   .Attr("alpha",
-        "Value of alpha",
-        AttrType::FLOAT)
+        "Value of alpha default to 0.2",
+        AttributeProto::FLOAT,
+        0.2f)
   .Attr("beta",
-        "Value of beta",
-        AttrType::FLOAT)
+        "Value of beta default to 0.5",
+        AttributeProto::FLOAT,
+        0.5f)
   .SetDoc(R"DOC(
 HardSigmoid takes one input data (Tensor<T>) and produces one output data
 (Tensor<T>) where the HardSigmoid function, y = max(0, min(1, alpha * x + beta)),
@@ -388,9 +362,7 @@ is applied to the tensor elementwise.
     .TypeConstraint("T", { "tensor(float16)", "tensor(float)", "tensor(double)" },
         "Constrain input and output types to float tensors.");
 
-OPERATOR_SCHEMA(Max)
-    .NumInputs(1, INT_MAX)
-    .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(Max)
     .AllowConsumed({{0, 0}})
     .SetDoc(R"DOC(
 Element-wise max of each of the input tensors. All inputs and outputs must
@@ -401,9 +373,7 @@ have the same shape and data type.
     .TypeConstraint("T", { "tensor(float16)", "tensor(float)", "tensor(double)" },
         "Constrain input and output types to float tensors.");
 
-OPERATOR_SCHEMA(Min)
-    .NumInputs(1, INT_MAX)
-    .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(Min)
     .AllowConsumed({{0, 0}})
     .SetDoc(R"DOC(
 Element-wise min of each of the input tensors. All inputs and outputs must
@@ -414,9 +384,7 @@ have the same shape and data type.
     .TypeConstraint("T", { "tensor(float16)", "tensor(float)", "tensor(double)" },
         "Constrain input and output types to float tensors.");
 
-OPERATOR_SCHEMA(Sum)
-    .NumInputs(1, INT_MAX)
-    .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(Sum)
     .AllowConsumed({{0, 0}})
     .SetDoc(R"DOC(
 Element-wise sum of each of the input tensors. All inputs and outputs must
@@ -427,9 +395,7 @@ have the same shape and data type.
     .TypeConstraint("T", { "tensor(float16)", "tensor(float)", "tensor(double)" },
         "Constrain input and output types to float tensors.");
 
-OPERATOR_SCHEMA(Mean)
-    .NumInputs(1, INT_MAX)
-    .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(Mean)
     .AllowConsumed({{0, 0}})
     .SetDoc(R"DOC(
 Element-wise mean of each of the input tensors. All inputs and outputs must
@@ -440,9 +406,7 @@ have the same shape and data type.
     .TypeConstraint("T", { "tensor(float16)", "tensor(float)", "tensor(double)" },
         "Constrain input and output types to float tensors.");
 
-OPERATOR_SCHEMA(Clip)
-    .NumInputs(1)
-    .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(Clip)
     .AllowConsumed({{0, 0}})
     .SetDoc(R"DOC(
 Clip operator limits the given input within an interval. The interval is
@@ -450,47 +414,39 @@ specified with arguments 'min' and 'max'. They default to
 numeric_limits::lowest() and numeric_limits::max() respectively.
 )DOC")
     .Attr("min", "Minimum value, under which element is replaced by min",
-          AttrType::FLOAT)
+          AttributeProto::FLOAT,
+          OPTIONAL)
     .Attr("max", "Maximum value, above which element is replaced by max",
-          AttrType::FLOAT)
+          AttributeProto::FLOAT,
+          OPTIONAL)
     .Input(0, "input", "Input tensor whose elements to be clipped", "T")
     .Output(0, "output", "Output tensor with clipped input elements", "T")
     .TypeConstraint("T", { "tensor(float16)", "tensor(float)", "tensor(double)" },
         "Constrain input and output types to float tensors.");
 
-OPERATOR_SCHEMA(Softmax)
-    .NumInputs(1)
-    .NumOutputs(1)
-    .FillUsing(SoftmaxFamilyDocGenerator("softmax"));
+ONNX_OPERATOR_SCHEMA(Softmax)
+    .FillUsing(SoftmaxFamilyDocGenerator("softmax", "normalized exponential"));
 
-OPERATOR_SCHEMA(LogSoftmax)
-    .NumInputs(1)
-    .NumOutputs(1)
-    .FillUsing(SoftmaxFamilyDocGenerator("logsoftmax"));
+ONNX_OPERATOR_SCHEMA(LogSoftmax)
+    .FillUsing(SoftmaxFamilyDocGenerator("logsoftmax", "log of softmax"));
 
-OPERATOR_SCHEMA(Hardmax)
-    .NumInputs(1)
-    .NumOutputs(1)
-    .FillUsing(SoftmaxFamilyDocGenerator("hardmax"));
+ONNX_OPERATOR_SCHEMA(Hardmax)
+    .FillUsing(SoftmaxFamilyDocGenerator("hardmax", "1 for the first maximum value, and 0 for all others"));
 
-OPERATOR_SCHEMA(Softsign)
-    .NumInputs(1)
-    .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(Softsign)
     .SetDoc(R"DOC(
-Calculates the softsign (x/1+|x|) of the given input tensor element-wise.
+Calculates the softsign (x/(1+|x|)) of the given input tensor element-wise.
 )DOC")
     .Input(0, "input", "1-D input tensor", "T")
     .Output(
         0,
         "output",
-        "The softsign (x/1+|x|) values of the input tensor computed element-wise",
+        "The softsign (x/(1+|x|)) values of the input tensor computed element-wise",
         "T")
     .TypeConstraint("T", { "tensor(float16)", "tensor(float)", "tensor(double)" },
         "Constrain input and output types to float tensors.");
 
-OPERATOR_SCHEMA(Softplus)
-    .NumInputs(1)
-    .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(Softplus)
     .SetDoc(R"DOC(
 Softplus takes one input data (Tensor<T>) and produces one output data
 (Tensor<T>) where the softplus function, y = ln(exp(x) + 1), is applied to
@@ -501,14 +457,12 @@ the tensor elementwise.
     .TypeConstraint("T", { "tensor(float16)", "tensor(float)", "tensor(double)" },
         "Constrain input and output types to float tensors.");
 
-OPERATOR_SCHEMA(Gemm)
-    .NumInputs(3)
-    .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(Gemm)
     .SetDoc(R"DOC(General Matrix multiplication:
 https://en.wikipedia.org/wiki/Basic_Linear_Algebra_Subprograms#Level_3
 Compute Y = alpha * A * B + beta * C, where input tensor A has dimension (M X K)
 , input tensor B has dimension (K X N), input tensor C and output tensor Y have
-dimension (M X N). 
+dimension (M X N).
 If attribute broadcast is non-zero, input tensor C will be broadcasted to match
 the dimension requirement. If A can be transposed before doing the computation
 if attribute transA is non-zero, same for B and transB.
@@ -522,24 +476,27 @@ if attribute transA is non-zero, same for B and transB.
         "Constrain input and output types to float tensors.")
     .Attr("transA",
           "Whether A should be transposed",
-          AttrType::INT)
+          AttributeProto::INT,
+          static_cast<int64_t>(0))
     .Attr("transB",
           "Whether B should be transposed",
-          AttrType::INT)
+          AttributeProto::INT,
+          static_cast<int64_t>(0))
     .Attr("broadcast",
           "Whether C should be broadcasted",
-          AttrType::INT)
+          AttributeProto::INT,
+          static_cast<int64_t>(0))
     .Attr("alpha",
           "Scalar multiplier for the product of input tensors A * B",
-          AttrType::FLOAT)
+          AttributeProto::FLOAT,
+          1.0f)
     .Attr("beta",
           "Scalar multiplier for input tensor C",
-          AttrType::FLOAT);
+          AttributeProto::FLOAT,
+          1.0f);
 
 
-OPERATOR_SCHEMA(MatMul)
-    .NumInputs(2)
-    .NumOutputs(1)
+ONNX_OPERATOR_SCHEMA(MatMul)
     .Input(0, "A", "N-dimensional matrix A", "T")
     .Input(1, "B", "N-dimensional matrix B", "T")
     .Output(0, "Y", "Matrix multiply results from A * B", "T")
@@ -548,3 +505,47 @@ OPERATOR_SCHEMA(MatMul)
     .SetDoc(R"DOC(
 Matrix product that behaves like numpy.matmul: https://docs.scipy.org/doc/numpy-1.13.0/reference/generated/numpy.matmul.html
 )DOC");
+
+ONNX_OPERATOR_SCHEMA(TopK)
+    .SetDoc(R"DOC(
+Retrieve the top-K elements along a specified axis. Given an input tensor of
+shape [a_1, a_2, ..., a_n, r] and integer argument k, return two outputs:
+  -Value tensor of shape [a_1, a_2, ..., a_{axis-1}, k, a_{axis+1}, ... a_n]
+    which contains the values of the top k elements along the specified axis
+  -Index tensor of shape [a_1, a_2, ..., a_{axis-1}, k, a_{axis+1}, ... a_n] which
+   contains the indices of the top k elements (original indices from the input
+   tensor).
+
+Given two equivalent values, this operator uses the indices along the axis  as
+ a tiebreaker. That is, the element with the lower index will appear first.
+)DOC")
+    .Input(0, "X", "Tensor of shape [a_1, a_2, ..., a_n, r]", "T")
+    .Output(
+        0,
+        "Values",
+        "Tensor of shape [a_1, a_2, ..., a_{axis-1}, k, a_{axis+1}, ... a_n] "
+        "containing top K values from the input tensor",
+        "T")
+    .Output(
+        1,
+        "Indices",
+        "Tensor of shape [a_1, a_2, ..., a_{axis-1}, k, a_{axis+1}, ... a_n] "
+        "containing the corresponding input tensor indices for the top K "
+        "values.",
+        "I")
+    .TypeConstraint(
+        "T",
+        {"tensor(float16)", "tensor(float)", "tensor(double)"},
+        "Constrain input and output types to float tensors.")
+    .TypeConstraint(
+        "I",
+        {"tensor(int64)", "tensor(int32)"},
+        "Constrain index tensor to integral types"
+    )
+    .Attr("k", "Number of top elements to retrieve", AttributeProto::INT, true)
+    .Attr(
+      "axis",
+      "Dimension on which to do the sort. Default -1, which indicates the last"
+      " axis",
+      AttributeProto::INT,
+      static_cast<int64_t>(-1));
